#ifndef RS12_H
#define RS12_H

/************************************************************************/
/* NAME                                                                 */
/*      rs12.h                                                          */
/* DESCRIPTION                                                          */
/*      Roland 12-bit-sampler disk format for the S-550, S-330, etc.).  */
/* AUTHOR                                                               */
/*      Copyright 1992 by Gary J. Murakami <gjm@research.att.com>       */
/*      V1.0 (4 May 1992) Freely distibutable for non-comercial use.    */
/*      All other rights reserved.                                      */
/************************************************************************/

#define SamplesPerSegment       12288

#ifndef Byte
#define Byte u_char
#endif

typedef struct systemProgram
{       char    data[0176000];          /* 0xFC00 == 64512 */
} SystemProgram;

typedef struct patchParameter
{       Byte    patchName[12];
        Byte    bendRange;
        Byte    dummy1[1];
        Byte    afterTouchSense;
        Byte    keyMode;
        Byte    velocitySwThreshold;
        Byte    toneToKey1[109];
        Byte    toneToKey2[109];
        Byte    copySource;
        Byte    octaveShift;
        Byte    outputLevel;
        Byte    dummy2[1];
        Byte    detune;
        Byte    velocityMixRatio;
        Byte    afterTouchAssign;
        Byte    keyAssign;
        Byte    outputAssign;
        Byte    dummy3[12];
} PatchParameter;

typedef struct functionParameter
{       Byte    masterTune;
        Byte    dummy1[13];
        Byte    audioTrig;
        Byte    dummy2[1];
        Byte    voiceMode;
        Byte    multiMidiRxCh[8];
        Byte    multiPatchNumber[8];
        Byte    dummy3[9];
        Byte    keyboardDisplay;
        Byte    multiLevel[8];
        Byte    diskLabel[60];
        Byte    dummy4[4];
        Byte    externalController;
        Byte    dummy5[140];
} FunctionParameter;

typedef struct midiParameter
{       Byte    dummy1[32];
        Byte    rxChannel[8];
        Byte    rxProgramChange[8];
        Byte    rxBender[8];
        Byte    rxModulation[8];
        Byte    rxHold[8];
        Byte    rxAfterTouch[8];
        Byte    rxVolume[8];
        Byte    rxBendRange[8];
        Byte    dummy2[1];
        Byte    systemExclusive;
        Byte    deviceId;
        Byte    dummy3[1];
        Byte    rxProgramChangeNumber[32];
        Byte    dummy4[124];
} MidiParameter;

typedef struct toneParameter
{       Byte    toneName[8];
        Byte    outputAssign;
        Byte    sourceTone;
        Byte    orgSubTone;
        Byte    samplingFrequency;
        Byte    origKeyNumber;
        Byte    waveBank;
        Byte    waveSegmentTop;
        Byte    waveSegmentLength;
        Byte    startPoint[3];
        Byte    endPoint[3];
        Byte    loopPoint[3];
        Byte    loopMode;
        Byte    tvaLfoDepth;
        Byte    dummy1[1];
        Byte    lfoRate;
        Byte    lfoSync;
        Byte    lfoDelay;
        Byte    dummy2[1];
        Byte    lfoMode;
        Byte    oscLfoDepth;
        Byte    lfoPolarity;
        Byte    lfoOffset;
        Byte    transpose;
        Byte    fineTune;
        Byte    tvfCutOff;
        Byte    tvfResonance;
        Byte    tvfKeyFollow;
        Byte    dummy3[1];
        Byte    tvfLfoDepth;
        Byte    tvfEgDepth;
        Byte    tvfEgPolarity;
        Byte    tvfLevelCurve;
        Byte    tvfKeyRateFollow;
        Byte    tvfVelocityRateFollow;
        Byte    dummy4[1];
        Byte    tvfSwitch;
        Byte    benderSwitch;
        Byte    tvaEnvSustainPoint;
        Byte    tvaEnvEndPoint;
        Byte    tvaEnvLevel1;
        Byte    tvaEnvRate1;
        Byte    tvaEnvLevel2;
        Byte    tvaEnvRate2;
        Byte    tvaEnvLevel3;
        Byte    tvaEnvRate3;
        Byte    tvaEnvLevel4;
        Byte    tvaEnvRate4;
        Byte    tvaEnvLevel5;
        Byte    tvaEnvRate5;
        Byte    tvaEnvLevel6;
        Byte    tvaEnvRate6;
        Byte    tvaEnvLevel7;
        Byte    tvaEnvRate7;
        Byte    tvaEnvLevel8;
        Byte    tvaEnvRate8;
        Byte    dummy5[1];
        Byte    tvaEnvKeyRate;
        Byte    level;
        Byte    envVelRate;
        Byte    recThreshold;
        Byte    recPreTrigger;
        Byte    recSamplingFrequency;
        Byte    recStartPoint[3];
        Byte    recEndPoint[3];
        Byte    recLoopPoint[3];
        Byte    zoomT;
        Byte    zoomL;
        Byte    copySource;
        Byte    loopTune;
        Byte    tvaLevelCurve;
        Byte    dummy6[12];
        Byte    loopLength[3];
        Byte    pitchFollow;
        Byte    envZoom;
        Byte    tvfEnvSustainPoint;
        Byte    tvfEnvEndPoint;
        Byte    tvfEnvLevel1;
        Byte    tvfEnvRate1;
        Byte    tvfEnvLevel2;
        Byte    tvfEnvRate2;
        Byte    tvfEnvLevel3;
        Byte    tvfEnvRate3;
        Byte    tvfEnvLevel4;
        Byte    tvfEnvRate4;
        Byte    tvfEnvLevel5;
        Byte    tvfEnvRate5;
        Byte    tvfEnvLevel6;
        Byte    tvfEnvRate6;
        Byte    tvfEnvLevel7;
        Byte    tvfEnvRate7;
        Byte    tvfEnvLevel8;
        Byte    tvfEnvRate8;
        Byte    afterTouchSwitch;
        Byte    dummy7[2];
} ToneParameter;

typedef struct toneList
{       Byte    toneName[8];
        Byte    unknown[8];
} ToneList;

typedef struct waveData
{       Byte    data[SamplesPerSegment*3/2];
} WaveData;

typedef struct disk
{       SystemProgram           systemProgram;          /*  0000 */
        PatchParameter          patchParameter[16];     /*  FC00 */
        FunctionParameter       functionParameter;      /* 10C00 */
        MidiParameter           midiParameter;          /* 10D00 */
        ToneParameter           toneParameter[32];      /* 10E00 */
        ToneList                toneList[32];           /* 11E00 */
        WaveData                waveDataA[18];          /* 12000 */
        WaveData                waveDataB[18];
} Disk;

#endif
